from hamcrest import assert_that
from allure_commons_test.report import has_test_case
from allure_commons_test.result import with_status
from allure_commons_test.result import has_status_details
from allure_commons_test.result import with_message_contains
from allure_commons_test.result import with_trace_contains
from allure_commons_test.container import has_container
from allure_commons_test.container import has_before


def test_xfail_with_run_false(executed_docstring_source):
    """
    >>> import pytest

    >>> @pytest.mark.xfail(run=False)
    ... def test_xfail_with_run_false_example():
    ...     pass
    """

    assert_that(executed_docstring_source.allure_report,
                has_test_case("test_xfail_with_run_false_example",
                              with_status("skipped"),
                              has_status_details(with_message_contains("Failed: [NOTRUN]")),
                              )
                )


def test_xfail_with_run_false_and_with_reason(executed_docstring_source):
    """
    >>> import pytest

    >>> @pytest.mark.xfail(run=False, reason="Some reason")
    ... def test_xfail_with_run_false_and_with_reason_example():
    ...     pass
    """

    assert_that(executed_docstring_source.allure_report,
                has_test_case("test_xfail_with_run_false_and_with_reason_example",
                              with_status("skipped"),
                              has_status_details(with_message_contains("Failed: [NOTRUN] Some reason"))
                              )
                )


def test_xfail_fixture(executed_docstring_source):
    """
    >>> import pytest

    >>> @pytest.fixture
    ... def broken_fixture():
    ...     raise NotImplementedError

    >>> @pytest.mark.xfail()
    ... def test_xfail_fixture_example(broken_fixture):
    ...     pass
    """

    assert_that(executed_docstring_source.allure_report,
                has_test_case("test_xfail_fixture_example",
                              with_status("skipped"),
                              has_status_details(with_message_contains("NotImplementedError"),
                                                 with_trace_contains("def broken_fixture():")
                                                 ),
                              has_container(executed_docstring_source.allure_report,
                                            has_before("broken_fixture",
                                                       with_status("broken"),
                                                       has_status_details(with_message_contains("NotImplementedError"),
                                                                          with_trace_contains("broken_fixture")
                                                                          ),
                                                       ),
                                            )
                              )
                )



import pytest
import allure_commons
from allure_commons.utils import escape_non_unicode_symbols
from allure_commons.utils import now
from allure_commons.utils import uuid4
from allure_commons.utils import represent
from allure_commons.utils import platform_label
from allure_commons.utils import host_tag, thread_tag
from allure_commons.reporter import AllureReporter
from allure_commons.model2 import TestStepResult, TestResult, TestBeforeResult, TestAfterResult
from allure_commons.model2 import TestResultContainer
from allure_commons.model2 import StatusDetails
from allure_commons.model2 import Parameter
from allure_commons.model2 import Label, Link
from allure_commons.model2 import Status
from allure_commons.types import LabelType, AttachmentType
from allure_pytest.utils import allure_description, allure_description_html
from allure_pytest.utils import allure_labels, allure_links, pytest_markers
from allure_pytest.utils import allure_full_name, allure_package, allure_name
from allure_pytest.utils import allure_suite_labels
from allure_pytest.utils import get_status, get_status_details
from allure_pytest.utils import get_outcome_status, get_outcome_status_details
from allure_pytest.utils import get_pytest_report_status
from allure_commons.utils import md5


class AllureListener(object):

    def __init__(self, config):
        self.config = config
        self.allure_logger = AllureReporter()
        self._cache = ItemCache()
        self._host = host_tag()
        self._thread = thread_tag()

    @allure_commons.hookimpl
    def start_step(self, uuid, title, params):
        parameters = [Parameter(name=name, value=value) for name, value in params.items()]
        step = TestStepResult(name=title, start=now(), parameters=parameters)
        self.allure_logger.start_step(None, uuid, step)

    @allure_commons.hookimpl
    def stop_step(self, uuid, exc_type, exc_val, exc_tb):
        self.allure_logger.stop_step(uuid,
                                     stop=now(),
                                     status=get_status(exc_val),
                                     statusDetails=get_status_details(exc_type, exc_val, exc_tb))

    @allure_commons.hookimpl
    def start_fixture(self, parent_uuid, uuid, name):
        after_fixture = TestAfterResult(name=name, start=now())
        self.allure_logger.start_after_fixture(parent_uuid, uuid, after_fixture)

    @allure_commons.hookimpl
    def stop_fixture(self, parent_uuid, uuid, name, exc_type, exc_val, exc_tb):
        self.allure_logger.stop_after_fixture(uuid,
                                              stop=now(),
                                              status=get_status(exc_val),
                                              statusDetails=get_status_details(exc_type, exc_val, exc_tb))

    @pytest.hookimpl(hookwrapper=True, tryfirst=True)
    def pytest_runtest_protocol(self, item, nextitem):
        uuid = self._cache.push(item.nodeid)
        test_result = TestResult(name=item.name, uuid=uuid, start=now(), stop=now())
        self.allure_logger.schedule_test(uuid, test_result)
        yield

    @pytest.hookimpl(hookwrapper=True)
    def pytest_runtest_setup(self, item):
        if not self._cache.get(item.nodeid):
            uuid = self._cache.push(item.nodeid)
            test_result = TestResult(name=item.name, uuid=uuid, start=now(), stop=now())
            self.allure_logger.schedule_test(uuid, test_result)

        yield

        uuid = self._cache.get(item.nodeid)
        test_result = self.allure_logger.get_test(uuid)
        for fixturedef in _test_fixtures(item):
            group_uuid = self._cache.get(fixturedef)
            if not group_uuid:
                group_uuid = self._cache.push(fixturedef)
                group = TestResultContainer(uuid=group_uuid)
                self.allure_logger.start_group(group_uuid, group)
            self.allure_logger.update_group(group_uuid, children=uuid)
        params = item.callspec.params if hasattr(item, 'callspec') else {}

        test_result.name = allure_name(item, params)
        full_name = allure_full_name(item)
        test_result.fullName = full_name
        test_result.historyId = md5(item.nodeid)
        test_result.testCaseId = md5(full_name)
        test_result.description = allure_description(item)
        test_result.descriptionHtml = allure_description_html(item)
        test_result.parameters.extend(
            [Parameter(name=name, value=represent(value)) for name, value in params.items()])

    @pytest.hookimpl(hookwrapper=True)
    def pytest_runtest_call(self, item):
        uuid = self._cache.get(item.nodeid)
        test_result = self.allure_logger.get_test(uuid)
        if test_result:
            test_result.start = now()
        yield
        if test_result:
            test_result.stop = now()

    @pytest.hookimpl(hookwrapper=True)
    def pytest_runtest_teardown(self, item):
        yield
        uuid = self._cache.get(item.nodeid)
        test_result = self.allure_logger.get_test(uuid)
        test_result.labels.extend([Label(name=name, value=value) for name, value in allure_labels(item)])
        test_result.labels.extend([Label(name=LabelType.TAG, value=value) for value in pytest_markers(item)])
        test_result.labels.extend([Label(name=name, value=value) for name, value in allure_suite_labels(item)])
        test_result.labels.append(Label(name=LabelType.HOST, value=self._host))
        test_result.labels.append(Label(name=LabelType.THREAD, value=self._thread))
        test_result.labels.append(Label(name=LabelType.FRAMEWORK, value='pytest'))
        test_result.labels.append(Label(name=LabelType.LANGUAGE, value=platform_label()))
        test_result.labels.append(Label(name='package', value=allure_package(item)))
        test_result.links.extend([Link(link_type, url, name) for link_type, url, name in allure_links(item)])

    @pytest.hookimpl(hookwrapper=True)
    def pytest_fixture_setup(self, fixturedef, request):
        fixture_name = fixturedef.argname

        container_uuid = self._cache.get(fixturedef)

        if not container_uuid:
            container_uuid = self._cache.push(fixturedef)
            container = TestResultContainer(uuid=container_uuid)
            self.allure_logger.start_group(container_uuid, container)

        self.allure_logger.update_group(container_uuid, start=now())

        before_fixture_uuid = uuid4()
        before_fixture = TestBeforeResult(name=fixture_name, start=now())
        self.allure_logger.start_before_fixture(container_uuid, before_fixture_uuid, before_fixture)

        outcome = yield

        self.allure_logger.stop_before_fixture(before_fixture_uuid,
                                               stop=now(),
                                               status=get_outcome_status(outcome),
                                               statusDetails=get_outcome_status_details(outcome))

        finalizers = getattr(fixturedef, '_finalizers', [])
        for index, finalizer in enumerate(finalizers):
            name = '{fixture}::{finalizer}'.format(fixture=fixturedef.argname,
                                                   finalizer=getattr(finalizer, "__name__", index))
            finalizers[index] = allure_commons.fixture(finalizer, parent_uuid=container_uuid, name=name)

    @pytest.hookimpl(hookwrapper=True)
    def pytest_fixture_post_finalizer(self, fixturedef):
        yield
        if hasattr(fixturedef, 'cached_result') and self._cache.get(fixturedef):
            container_uuid = self._cache.pop(fixturedef)
            self.allure_logger.stop_group(container_uuid, stop=now())

    @pytest.hookimpl(hookwrapper=True)
    def pytest_runtest_makereport(self, item, call):
        uuid = self._cache.get(item.nodeid)

        report = (yield).get_result()

        test_result = self.allure_logger.get_test(uuid)
        status = get_pytest_report_status(report)
        status_details = None

        if call.excinfo:
            status_details = StatusDetails(
                message=escape_non_unicode_symbols(call.excinfo.exconly()),
                trace=escape_non_unicode_symbols(report.longreprtext))
            if (status != Status.SKIPPED
                and not (call.excinfo.errisinstance(AssertionError)
                         or call.excinfo.errisinstance(pytest.fail.Exception))):
                status = Status.BROKEN

        if status == Status.PASSED and hasattr(report, 'wasxfail'):
            reason = report.wasxfail
            message = 'XPASS {reason}'.format(reason=reason) if reason else 'XPASS'
            status_details = StatusDetails(message=message)

        if report.when == 'setup':
            test_result.status = status
            test_result.statusDetails = status_details

        if report.when == 'call':
            if test_result.status == Status.PASSED:
                test_result.status = status
                test_result.statusDetails = status_details

        if report.when == 'teardown':
            if status in (Status.FAILED, Status.BROKEN) and test_result.status == Status.PASSED:
                test_result.status = status
                test_result.statusDetails = status_details

            if self.config.option.attach_capture:
                if report.caplog:
                    self.attach_data(report.caplog, "log", AttachmentType.TEXT, None)
                if report.capstdout:
                    self.attach_data(report.capstdout, "stdout", AttachmentType.TEXT, None)
                if report.capstderr:
                    self.attach_data(report.capstderr, "stderr", AttachmentType.TEXT, None)

    @pytest.hookimpl(hookwrapper=True)
    def pytest_runtest_logfinish(self, nodeid, location):
        yield
        uuid = self._cache.pop(nodeid)
        if uuid:
            self.allure_logger.close_test(uuid)

    @allure_commons.hookimpl
    def attach_data(self, body, name, attachment_type, extension):
        self.allure_logger.attach_data(uuid4(), body, name=name, attachment_type=attachment_type, extension=extension)

    @allure_commons.hookimpl
    def attach_file(self, source, name, attachment_type, extension):
        self.allure_logger.attach_file(uuid4(), source, name=name, attachment_type=attachment_type, extension=extension)

    @allure_commons.hookimpl
    def add_title(self, test_title):
        test_result = self.allure_logger.get_test(None)
        if test_result:
            test_result.name = test_title

    @allure_commons.hookimpl
    def add_description(self, test_description):
        test_result = self.allure_logger.get_test(None)
        if test_result:
            test_result.description = test_description

    @allure_commons.hookimpl
    def add_description_html(self, test_description_html):
        test_result = self.allure_logger.get_test(None)
        if test_result:
            test_result.descriptionHtml = test_description_html

    @allure_commons.hookimpl
    def add_link(self, url, link_type, name):
        test_result = self.allure_logger.get_test(None)
        if test_result:
            pattern = dict(self.config.option.allure_link_pattern).get(link_type, u'{}')
            link_url = pattern.format(url)
            new_link = Link(link_type, link_url, link_url if name is None else name)
            for link in test_result.links:
                if link.url == new_link.url:
                    return
            test_result.links.append(new_link)

    @allure_commons.hookimpl
    def add_label(self, label_type, labels):
        test_result = self.allure_logger.get_test(None)
        for label in labels if test_result else ():
            test_result.labels.append(Label(label_type, label))


class ItemCache(object):

    def __init__(self):
        self._items = dict()

    def get(self, _id):
        return self._items.get(str(_id))

    def push(self, _id):
        return self._items.setdefault(str(_id), uuid4())

    def pop(self, _id):
        return self._items.pop(str(_id), None)


def _test_fixtures(item):
    fixturemanager = item.session._fixturemanager
    fixturedefs = []

    if hasattr(item, "fixturenames"):
        for name in item.fixturenames:
            fixturedef = fixturemanager.getfixturedefs(name, item.nodeid)
            if fixturedef:
                fixturedefs.append(fixturedef[-1])

    return fixturedefs



""" ./examples/label/custom/custom_label.rst """

from hamcrest import assert_that
from allure_commons_test.report import has_test_case
from allure_commons_test.label import has_label


def test_custom_label(executed_docstring_path):
    assert_that(executed_docstring_path.allure_report,
                has_test_case("test_custom_label",
                              has_label("Application", "desktop"),
                              has_label("Application", "mobile")
                              )
                )



""" ./examples/label/bdd/select_tests_by_bdd.rst """

import pytest
from hamcrest import assert_that, only_contains, any_of, ends_with


@pytest.mark.parametrize(
    ["options", "expected_tests"],
    [
        ({"epics": ["Another Epic"]},
         [
             "test_with_another_epic_feature_story"
         ]),

        ({"features": ["My Feature"]},
         [
             "test_with_epic_feature_story",
             "test_with_epic_feature"
         ]),

        ({"stories": ["My Story", "Another Story"]},
         [
             "test_with_epic_feature_story",
             "test_with_another_epic_feature_story"
         ]),

        ({"stories": ["My Story"], "epics": ["Another Epic"]},
         [
             "test_with_epic_feature_story",
             "test_with_another_epic_feature_story"
         ])

    ]
)
def test_select_by_bdd_label(allured_testdir, options, expected_tests):
    allured_testdir.parse_docstring_path()

    params = []

    for key in options.keys():
        params.append("--allure-{key}".format(key=key))
        params.append(",".join(options[key]))

    allured_testdir.run_with_allure(*params)
    test_cases = [test_case["fullName"] for test_case in allured_testdir.allure_report.test_cases]

    assert_that(test_cases, only_contains(
        any_of(
            *[ends_with(name) for name in expected_tests]
        )
    ))



from hamcrest import assert_that
from allure_commons_test.report import has_test_case
from allure_commons_test.label import has_severity


def test_severity(executed_docstring_path):
    """ ./examples/label/severity/severity.rst """

    assert_that(executed_docstring_path.allure_report,
                has_test_case("test_severity",
                              has_severity("minor"),
                              )
                )



import allure
import pytest
from hamcrest import assert_that, has_entry, greater_than, all_of
from allure_commons_test.report import has_test_case
from allure_commons.utils import now


snippets = [
    "pass",
    "assert False",
    "raise(RuntimeError())",
    "pytest.skip()",
    "pytest.fail()",
    "pytest.xfail()",
    pytest.param("pytest.exit('msg')", marks=pytest.mark.skip)
]


@pytest.mark.parametrize("snipped", snippets)
def test_duration(allured_testdir, snipped):
    allured_testdir.testdir.makepyfile("""
        def test_duration_example():
            {snipped}
    """.format(snipped=snipped))

    timestamp = now()
    allured_testdir.run_with_allure()

    assert_that(allured_testdir.allure_report,
                has_test_case("test_duration_example",
                              all_of(
                                  has_entry("start", greater_than(timestamp)),
                                  has_entry("stop", greater_than(timestamp))
                              ))
                )


@allure.issue("244")
@pytest.mark.parametrize("snipped", snippets)
def test_with_fixture_duration(allured_testdir, snipped):
    allured_testdir.testdir.makepyfile("""
        import pytest

        @pytest.fixture
        def fixture():
            {snipped}

        def test_with_fixture_duration_example(fixture):
            pass
    """.format(snipped=snipped))

    timestamp = now()
    allured_testdir.run_with_allure()

    assert_that(allured_testdir.allure_report,
                has_test_case("test_with_fixture_duration_example",
                              all_of(
                                  has_entry("start", greater_than(timestamp)),
                                  has_entry("stop", greater_than(timestamp))
                              ))
                )


@allure.issue("244")
@pytest.mark.parametrize("snipped", snippets)
def test_with_fixture_finalizer_duration(allured_testdir, snipped):
    allured_testdir.testdir.makepyfile("""
        import pytest

        @pytest.fixture
        def fixture(request):
            def finalizer():
                {snipped}
            request.addfinalizef(finalizer)

        def test_with_fixture_finalizer_duration(fixture):
            pass
    """.format(snipped=snipped))

    timestamp = now()
    allured_testdir.run_with_allure()

    assert_that(allured_testdir.allure_report,
                has_test_case("test_with_fixture_finalizer_duration",
                              all_of(
                                  has_entry("start", greater_than(timestamp)),
                                  has_entry("stop", greater_than(timestamp))
                              ))
                )



from hamcrest import assert_that
from allure_commons_test.report import has_test_case
from allure_commons_test.result import with_status
from allure_commons_test.result import has_status_details
from allure_commons_test.result import with_message_contains
from allure_commons_test.result import with_trace_contains
from allure_commons_test.container import has_container
from allure_commons_test.container import has_before


def test_failed_fixture(executed_docstring_source):
    """
    >>> import pytest

    >>> @pytest.fixture
    ... def failed_fixture():
    ...     assert False

    >>> def test_failed_fixture_example(failed_fixture):
    ...     pass
    """

    assert_that(executed_docstring_source.allure_report,
                has_test_case("test_failed_fixture_example",
                              with_status("failed"),
                              has_status_details(with_message_contains("AssertionError"),
                                                 with_trace_contains("def failed_fixture():")
                                                 ),
                              has_container(executed_docstring_source.allure_report,
                                            has_before("failed_fixture",
                                                       with_status("failed"),
                                                       has_status_details(with_message_contains("AssertionError"),
                                                                          with_trace_contains("failed_fixture")
                                                                          ),
                                                       ),
                                            )
                              )
                )


def test_broken_fixture(executed_docstring_source):
    """
    >>> import pytest

    >>> @pytest.fixture
    ... def broken_fixture():
    ...     raise IndexError

    >>> def test_broken_fixture_example(broken_fixture):
    ...     pass
    """

    assert_that(executed_docstring_source.allure_report,
                has_test_case("test_broken_fixture_example",
                              with_status("broken"),
                              has_status_details(with_message_contains("IndexError"),
                                                 with_trace_contains("def broken_fixture():")
                                                 ),
                              has_container(executed_docstring_source.allure_report,
                                            has_before("broken_fixture",
                                                       with_status("broken"),
                                                       has_status_details(with_message_contains("IndexError"),
                                                                          with_trace_contains("broken_fixture")
                                                                          ),
                                                       ),
                                            )
                              )
                )


def test_skip_fixture(executed_docstring_source):
    """
    >>> import pytest

    >>> @pytest.fixture
    ... def skip_fixture():
    ...     pytest.skip()

    >>> def test_skip_fixture_example(skip_fixture):
    ...     pass
    """

    assert_that(executed_docstring_source.allure_report,
                has_test_case("test_skip_fixture_example",
                              with_status("skipped"),
                              has_status_details(with_message_contains("Skipped: <Skipped instance>")),
                              has_container(executed_docstring_source.allure_report,
                                            has_before("skip_fixture",
                                                       with_status("skipped"),
                                                       has_status_details(
                                                           with_message_contains("Skipped: <Skipped instance>"),
                                                           with_trace_contains("skip_fixture")
                                                       ),
                                                       ),
                                            )
                              )
                )


def test_pytest_fail_fixture(executed_docstring_source):
    """
    >>> import pytest

    >>> @pytest.fixture
    ... def pytest_fail_fixture():
    ...     pytest.fail()

    >>> def test_pytest_fail_fixture_example(pytest_fail_fixture):
    ...     pass
    """

    assert_that(executed_docstring_source.allure_report,
                has_test_case("test_pytest_fail_fixture_example",
                              with_status("failed"),
                              has_status_details(with_message_contains("Failed: <Failed instance>"),
                                                 with_trace_contains("def pytest_fail_fixture():")
                                                 ),
                              has_container(executed_docstring_source.allure_report,
                                            has_before("pytest_fail_fixture",
                                                       with_status("failed"),
                                                       has_status_details(
                                                           with_message_contains("Failed: <Failed instance>"),
                                                           with_trace_contains("pytest_fail_fixture")
                                                       ),
                                                       ),
                                            )
                              )
                )


def test_pytest_fail_with_reason_fixture(executed_docstring_source):
    """
    >>> import pytest

    >>> @pytest.fixture
    ... def pytest_fail_with_reason_fixture():
    ...     pytest.fail("Fail message")

    >>> def test_pytest_fail_with_reason_fixture_example(pytest_fail_with_reason_fixture):
    ...     pass
    """

    assert_that(executed_docstring_source.allure_report,
                has_test_case("test_pytest_fail_with_reason_fixture_example",
                              with_status("failed"),
                              has_status_details(with_message_contains("Fail message"),
                                                 with_trace_contains("def pytest_fail_with_reason_fixture():")
                                                 ),
                              has_container(executed_docstring_source.allure_report,
                                            has_before("pytest_fail_with_reason_fixture",
                                                       with_status("failed"),
                                                       has_status_details(with_message_contains("Fail message"),
                                                                          with_trace_contains(
                                                                              "pytest_fail_with_reason_fixture")
                                                                          ),
                                                       ),
                                            )
                              )
                )



""" ./examples/label/suite/module_level_custom_suite.rst """

from hamcrest import assert_that
from allure_commons_test.report import has_test_case
from allure_commons_test.label import has_suite


def test_module_custom_suite(executed_docstring_path):
    assert_that(executed_docstring_path.allure_report,
                has_test_case("test_module_level_custom_suite",
                              has_suite("module level suite name"),
                              )
                )



from hamcrest import assert_that
from allure_commons_test.report import has_test_case
from allure_commons_test.result import with_status
from allure_commons_test.result import has_status_details
from allure_commons_test.result import with_message_contains
from allure_commons_test.result import with_trace_contains


def test_xfail(executed_docstring_source):
    """
    >>> import pytest

    >>> @pytest.mark.xfail()
    ... def test_xfail_example():
    ...     assert False

    """

    assert_that(executed_docstring_source.allure_report,
                has_test_case("test_xfail_example",
                              with_status("skipped"),
                              has_status_details(with_message_contains("AssertionError"),
                                                 with_trace_contains("def test_xfail_example():")
                                                 )
                              )
                )


def test_xfail_raise_mentioned_exception(executed_docstring_source):
    """
    >>> import pytest

    >>> @pytest.mark.xfail(raises=AssertionError)
    ... def test_xfail_raise_mentioned_exception_example():
    ...     assert False

    """

    assert_that(executed_docstring_source.allure_report,
                has_test_case("test_xfail_raise_mentioned_exception_example",
                              with_status("skipped"),
                              has_status_details(with_message_contains("AssertionError"),
                                                 with_trace_contains(
                                                     "def test_xfail_raise_mentioned_exception_example():")
                                                 )
                              )
                )


def test_xfail_raise_not_mentioned_exception(executed_docstring_source):
    """
    >>> import pytest

    >>> @pytest.mark.xfail(raises=AssertionError)
    ... def test_xfail_raise_not_mentioned_exception_example():
    ...     raise ZeroDivisionError
    """

    assert_that(executed_docstring_source.allure_report,
                has_test_case("test_xfail_raise_not_mentioned_exception_example",
                              with_status("broken"),
                              has_status_details(with_message_contains("ZeroDivisionError"),
                                                 with_trace_contains(
                                                     "def test_xfail_raise_not_mentioned_exception_example():")
                                                 )
                              )
                )


def test_xfail_do_not_raise_mentioned_exception(executed_docstring_source):
    """
    >>> import pytest

    >>> @pytest.mark.xfail(raises=AssertionError)
    ... def test_xfail_do_not_raise_mentioned_exception_example():
    ...     pass
    """

    assert_that(executed_docstring_source.allure_report,
                has_test_case("test_xfail_do_not_raise_mentioned_exception_example",
                              with_status("passed"),
                              has_status_details(with_message_contains("XPASS"),
                                                 )
                              )
                )


def test_xfail_with_reason_do_not_raise_mentioned_exception(executed_docstring_source):
    """
    >>> import pytest

    >>> @pytest.mark.xfail(raises=AssertionError, reason="Some reason")
    ... def test_xfail_with_reason_do_not_raise_mentioned_exception_example():
    ...     pass
    """

    assert_that(executed_docstring_source.allure_report,
                has_test_case("test_xfail_with_reason_do_not_raise_mentioned_exception_example",
                              with_status("passed"),
                              has_status_details(with_message_contains("XPASS Some reason"),
                                                 )
                              )
                )



from hamcrest import assert_that
from allure_commons_test.report import has_test_case
from allure_commons_test.result import has_history_id


def test_history_id(executed_docstring_source):
    """
    >>> def test_history_id_example():
    ...     assert True
    """

    assert_that(executed_docstring_source.allure_report,
                has_test_case("test_history_id_example",
                              has_history_id()
                              )
                )


def test_history_id_for_skipped(executed_docstring_source):
    """
    >>> import pytest

    >>> @pytest.mark.skip
    ... def test_history_id_for_skipped_example():
    ...     assert True
    """

    assert_that(executed_docstring_source.allure_report,
                has_test_case("test_history_id_for_skipped_example",
                              has_history_id()
                              )
                )



import pytest
from hamcrest import assert_that
from allure_commons_test.report import has_test_case
from allure_commons_test.container import has_container
from allure_commons_test.container import has_before
from allure_commons_test.result import has_parameter


def params_name(request):
    node_id = request.node.nodeid
    _, name = node_id.rstrip("]").split("[")
    return name


@pytest.mark.parametrize("param", [True, False])
def test_function_scope_parametrized_fixture(param, executed_docstring_source):
    """
    >>> import pytest

    >>> @pytest.fixture(params=[True, False])
    ... def parametrized_fixture(request):
    ...     pass

    >>> def test_function_scope_parametrized_fixture_example(parametrized_fixture):
    ...     pass
    """
    assert_that(executed_docstring_source.allure_report,
                has_test_case("test_function_scope_parametrized_fixture_example[{param}]".format(param=param),
                              has_parameter("parametrized_fixture", str(param)),
                              has_container(executed_docstring_source.allure_report,
                                            has_before("parametrized_fixture")
                                            )
                              )
                )


@pytest.mark.parametrize("param", [True, False], ids=["param_true", "param_false"])
def test_function_scope_parametrized_fixture_with_ids(param, executed_docstring_source, request):
    """
    >>> import pytest

    >>> @pytest.fixture(params=[True, False], ids=["param_true", "param_false"])
    ... def parametrized_fixture(request):
    ...     pass

    >>> def test_function_scope_parametrized_fixture_with_ids_example(parametrized_fixture):
    ...     pass
    """

    test_name = "test_function_scope_parametrized_fixture_with_ids_example[{params_name}]".format(
        params_name=params_name(request))

    assert_that(executed_docstring_source.allure_report,
                has_test_case(test_name,
                              has_parameter("parametrized_fixture", str(param)),
                              has_container(executed_docstring_source.allure_report,
                                            has_before("parametrized_fixture")
                                            )
                              )
                )



import allure
from hamcrest import assert_that, not_
from allure_commons_test.report import has_test_case
from allure_commons_test.result import has_step
from allure_commons_test.container import has_container
from allure_commons_test.container import has_before


def test_step_from_init_py(allured_testdir):
    allured_testdir.testdir.makepyfile(__init__="""
        import allure

        @allure.step("function in __init__ marked as step")
        def step_from__init__():
            pass
    """)

    allured_testdir.testdir.makepyfile("""
        from . import step_from__init__

        def test_step_from_init_py_example():
            step_from__init__()
    """)

    allured_testdir.run_with_allure()

    assert_that(allured_testdir.allure_report,
                has_test_case("test_step_from_init_py_example",
                              has_step("function in __init__ marked as step")
                              )
                )


def test_fixture_with_step_from_conftest(allured_testdir):
    allured_testdir.testdir.makeconftest("""
        import allure
        import pytest

        @allure.step("step in conftest.py")
        def conftest_step():
            pass


        @pytest.fixture
        def fixture_with_conftest_step():
            conftest_step()
    """)

    allured_testdir.testdir.makepyfile("""
        def test_fixture_with_step_from_conftest_example(fixture_with_conftest_step):
            pass
    """)

    allured_testdir.run_with_allure()

    assert_that(allured_testdir.allure_report,
                has_test_case("test_fixture_with_step_from_conftest_example",
                              has_container(allured_testdir.allure_report,
                                            has_before("fixture_with_conftest_step",
                                                       has_step("step in conftest.py")
                                                       )
                                            )
                              )
                )


@allure.issue("232")
def test_call_decorated_as_step_function(executed_docstring_source):
    """
    >>> import allure

    >>> with allure.step("step outside"):
    ...     pass

    >>> def test_call_decorated_as_step_function_example():
    ...     pass
    """

    assert_that(executed_docstring_source.allure_report,
                has_test_case("test_call_decorated_as_step_function_example",
                              not_(has_step("step outside"))
                              )
                )



import allure
from hamcrest import assert_that
from allure_commons_test.report import has_test_case
from allure_commons_test.container import has_container
from allure_commons_test.container import has_before


@allure.feature("Fixture")
def test_yield_fixture(executed_docstring_source):
    """
    >>> import pytest

    >>> @pytest.fixture
    ... def yield_fixture():
    ...     pass

    >>> def test_yield_fixture_example(yield_fixture):
    ...     pass
    """

    assert_that(executed_docstring_source.allure_report,
                has_test_case("test_yield_fixture_example",
                              has_container(executed_docstring_source.allure_report,
                                            has_before("yield_fixture")
                                            ),
                              )
                )



import os,sys
from setuptools import setup
from pkg_resources import require, DistributionNotFound, VersionConflict

try:
    require('pytest-allure-adaptor')
    print("""
    You have pytest-allure-adaptor installed.
    You need to remove pytest-allure-adaptor from your site-packages
    before installing allure-pytest, or conflicts may result.
    """)
    sys.exit()
except (DistributionNotFound, VersionConflict):
    pass

PACKAGE = "allure-pytest"

classifiers = [
    'Development Status :: 5 - Production/Stable',
    'Framework :: Pytest',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: Apache Software License',
    'Topic :: Software Development :: Quality Assurance',
    'Topic :: Software Development :: Testing',
]

setup_requires = [
    "setuptools_scm"
]


install_requires = [
    "pytest>=4.5.0",
    "six>=1.9.0",
]


def prepare_version():
    from setuptools_scm import get_version
    configuration = {"root": "..",  "relative_to": __file__}
    version = get_version(**configuration)
    install_requires.append("allure-python-commons=={version}".format(version=version))
    return configuration


def get_readme(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


def main():
    setup(
        name=PACKAGE,
        use_scm_version=prepare_version,
        description="Allure pytest integration",
        url="https://github.com/allure-framework/allure-python",
        author="QAMetaSoftware, Stanislav Seliverstov",
        author_email="sseliverstov@qameta.io",
        license="Apache-2.0",
        classifiers=classifiers,
        keywords="allure reporting pytest",
        long_description=get_readme('README.rst'),
        packages=["allure_pytest"],
        package_dir={"allure_pytest": "src"},
        entry_points={"pytest11": ["allure_pytest = allure_pytest.plugin"]},
        setup_requires=setup_requires,
        install_requires=install_requires
    )

if __name__ == '__main__':
    main()




import pytest
import json
import os
from hamcrest import assert_that, contains_inanyorder, ends_with


@pytest.mark.parametrize(
    ["planned_tests", "expected_tests"],
    [
        # by ids only
        (
            [{"id": 1}, {"id": 2}],
            ["test_number_one", "test_number_two"]
        ),

        # by ids for multiply decorated test
        (
            [{"id": 1}, {"id": 3}, {"id": 4}],
            ["test_number_one", "test_number_three"]
        ),

        # by wrong id
        (
            [{"id": 1234}],
            []
        ),

        # by selectors only
        (
            [{"selector": "test_number_one"}, {"selector": "test_number_three"}],
            ["test_number_one", "test_number_three"]
        ),

        # by selector for not decorated with id test
        (
            [{"selector": "test_without_number"}],
            ["test_without_number"]
        ),

        # by id and selector for same test
        (
            [{"id": 2, "selector": "test_number_two"}],
            ["test_number_two"]
        ),

        # by wrong selector
        (
            [{"selector": "test_without_never"}],
            []
        ),

        # without plan
        (
            None,
            ["test_number_one", "test_number_two", "test_number_three", "test_without_number"]
        ),
    ]
)
def test_select_by_testcase_id_test(planned_tests, expected_tests, allured_testdir, request):
    """
    >>> import allure

    >>> @allure.id("1")
    ... def test_number_one():
    ...     pass

    >>> @allure.id("2")
    ... def test_number_two():
    ...     pass

    >>> @allure.id("3")
    ... @allure.id("4")
    ... def test_number_three():
    ...     pass

    >>> def test_without_number():
    ...     pass
    """

    root_dir = request.config.rootdir.strpath
    test_dir = allured_testdir.testdir.tmpdir.strpath.replace(root_dir, "")
    full_name_base_template = "{base}.test_select_by_testcase_id_test".format(
        base=test_dir.strip(os.sep).replace(os.sep, "."))

    if planned_tests:
        for item in planned_tests:
            if "selector" in item:
                item["selector"] = "{base}#{name}".format(base=full_name_base_template, name=item["selector"])

        testplan = {"tests": planned_tests}
        py_path = allured_testdir.testdir.makefile(".json", json.dumps(testplan))
        os.environ["AS_TESTPLAN_PATH"] = py_path.strpath
    else:
        os.environ.pop("AS_TESTPLAN_PATH", None)

    allured_testdir.parse_docstring_source()
    allured_testdir.run_with_allure()

    executed_full_names = [test_case["fullName"] for test_case in allured_testdir.allure_report.test_cases]

    assert_that(executed_full_names, contains_inanyorder(*[ends_with(name) for name in expected_tests]))



""" ./examples/label/severity/module_severity.rst """

from hamcrest import assert_that
from allure_commons_test.report import has_test_case
from allure_commons_test.label import has_severity


def test_not_decorated_function(executed_docstring_path):
    assert_that(executed_docstring_path.allure_report,
                has_test_case("test_not_decorated_function",
                              has_severity("trivial")
                              )
                )


def test_decorated_function(executed_docstring_path):
    assert_that(executed_docstring_path.allure_report,
                has_test_case("test_decorated_function",
                              has_severity("minor")
                              )
                )


def test_method_of_not_decorated_class(executed_docstring_path):
    assert_that(executed_docstring_path.allure_report,
                has_test_case("test_method_of_not_decorated_class",
                              has_severity("trivial")
                              )
                )


def test_method_of_decorated_class(executed_docstring_path):
    assert_that(executed_docstring_path.allure_report,
                has_test_case("test_method_of_decorated_class",
                              has_severity("normal")
                              )
                )



import allure
from hamcrest import assert_that
from allure_commons_test.report import has_test_case
from allure_commons_test.container import has_container
from allure_commons_test.container import has_before, has_after


@allure.feature("Fixture")
@allure.story("Fixture finalizer")
def test_fixture_finalizer(executed_docstring_source):
    """
    >>> import pytest

    Let"s define fixture with finalizer:
    >>> @pytest.fixture
    ... def fixture_with_finalizer(request):
    ...     def finalizer():
    ...         pass
    ...     request.addfinalizer(finalizer)

    For next test, allure will report fixture finalizer in TearDown section
    >>> def test_fixture_with_finalizer_example(fixture_with_finalizer):
    ...     pass
    """

    assert_that(executed_docstring_source.allure_report,
                has_test_case("test_fixture_with_finalizer_example",
                              has_container(executed_docstring_source.allure_report,
                                            has_before("fixture_with_finalizer"),
                                            has_after("{fixture}::{finalizer}".format(fixture="fixture_with_finalizer",
                                                                                      finalizer="finalizer")
                                                      )
                                            )
                              )
                )


@allure.feature("Fixture")
@allure.story("Fixture finalizer")
def test_fixture_finalizers(executed_docstring_source):
    """
    >>> import pytest

    In pytest, you can define several finalizers for one fixture, like this:
    >>> @pytest.fixture
    ... def fixture_with_finalizers(request):
    ...     def first_finalizer():
    ...         pass
    ...     request.addfinalizer(first_finalizer)
    ...
    ...     def second_finalizer():
    ...         pass
    ...     request.addfinalizer(second_finalizer)

    Of course, allure will report all of them
    >>> def test_fixture_with_finalizers_example(fixture_with_finalizers):
    ...     pass
    """

    assert_that(executed_docstring_source.allure_report,
                has_test_case("test_fixture_with_finalizers_example",
                              has_container(executed_docstring_source.allure_report,
                                            has_before("fixture_with_finalizers"),
                                            has_after("{fixture}::{finalizer}".format(fixture="fixture_with_finalizers",
                                                                                      finalizer="first_finalizer")
                                                      ),
                                            has_after("{fixture}::{finalizer}".format(fixture="fixture_with_finalizers",
                                                                                      finalizer="second_finalizer"),
                                                      )
                                            )
                              )
                )
